; Authors: Titan, BoBo, Metaxal, Scribbly
; GNU General Public License 3.0 or higher <http://www.gnu.org/licenses/gpl-3.0.txt>
thisScript_Version=0.19.0
/*
   original topic:
   http://www.autohotkey.com/forum/viewtopic.php?t=23671

   Version history:

		- 0.19.0s 2009.10.09 (Scribbly)
		    - fixed Toggle CoordMode Mouse (not setting mode correctly)
		    - Added Recording Marker
		    - help file
		- 0.18s (Scribbly)
		    - corrected russian code number
		    - added Recording window + hotkeys (Single Function keys work best)
		    - temp.ahk sent to A_temp; renamed to scriptname.temp.ahk
		    - Recorder HotKeys set on "Start"
    - 0.17m (Metaxal)
        - bugfix: moverel now moves the mouse relatively to the initial mouse coordinates
        - added "WAIT!" message
        - (bug) added var gui_windowrel, to move in Coordmode Relative, NOT YET FUNCTIONAL!
            added gui control
            (Russian and German traductions for this are approximate)
    - 0.16m (Metaxal)
        - added: Russian text by Anton Bukov
   - 0.15m (Metaxal)
      - added: ExitApp key in generated scripts (the same key as the one used to stop recording, which makes sense!)
      - added NumpadEnter as bug key
      - added Replay button, script saved by default in temp.ahk
   - 0.14m (Metaxal)
      - fixed: buggy keys (Delete, Home, PgUp, ...) not logged
      - fixed: recording mouse moves was not functional?
      - not tested: relative mouse moves
   - 0.13m (Metaxal)
      - fixed: mouse clicks not recorded relative to screen...
      - Using VK codes instead of explicit symbols
         -> no problem whatever the foreign keyboard?
         -> can handle mysterious keys !
         -> handles mouse up and down + joysticks (?)
   - 0.12m (Metaxal)
      - french translation
      - removed consolidate, added add_log
      -> no need for #Include grep.ahk anymore. Multiple "Sends" are still merged, but "aaaa" is no more compressed
      -> also corrects the "Sl{e 2}p" error
      - added WinWait, IfWinActive and WinWaitActive
      - F11 and F12 for start/stop prefered to avoid some bugs with combined hotkeys
      - fixed: WinActivate not detected (wParam=32772)
      - fixed: "Send, " without parameter was added at end of script
      - fixed: mouse cordinates not relative or absolute
      - fixed(?): pressing Shift or Control at beginning of script would go to some other hotkey label
      (sometimes stop, sometimes start, or even exit when *~Esc is active)
      resolved by placing the call to hotkeys() before initializing the start and stop hotkeys
      Not sure this is completely fixed
      - fixed: wrong Sleep times
      - fixed: Sleep not initialized at each new recording
      - NOT fixed: keys are sometimes missed, and sometimes added??

   - 0.11a
      Original version by Titan, german translations by BoBo
*/

/*
   TODO:
   - Add a tooltip in the generated script to show when the script is playng or is ended.
*/

#SingleInstance ignore
#NoEnv
Menu, tray, icon, %A_ScriptDir%\icons\Recorder.ico
#NoTrayIcon

#Include %A_ScriptDir% ; Metaxal: Ensures a consistent #Include directory
#Include includes\Anchor.ahk
; #Include grep.ahk ; Metaxal: not needed anymore
#Include %A_ScriptDir%\includes\ScriptEditorFunctions.ahk

SetWorkingDir %A_ScriptDir%  ; Ensures a consistent starting directory.

SetBatchLines, -1
CoordMode, ToolTip, Screen
CoordMode, Mouse, Screen

lang_table =
( Join`n `
en,de,fr,ru
AMPL Recorder,AMPL Recorder, AMPL Enregistreur,????????? ???????? Titan-?
Recording options,Aufnahmeoptionen,Options d'enregistrement,????? ??????
&Time intervals,&Zeitintervalle,&Intervalles de temps,????????? ?????????
&Window actions,&Fensteraktionen,&Gestion des fenetres,???????? ??? ??????
&Use control commands,&Control-Befehle,&Commandes de controle,???????? ??? ?????????? ??????????
&Keyboard actions,&Tastaturaktionen,C&lavier,??????? ??????
Mouse c&licks,&Mausklicks,Clics &souris,????? ??????
Mouse &Movements,Mouse &Bewegungen,Mouse &Mouvements,????????
Relative,Relative,Relative,??
Hotkeys,Hotkeys,Hotkeys,??????? ???????
S&tart,St&art,&Debut,?????
St&op,St&op,&Fin,????
&Save As...,&Speichern unter...,&Enregistrer sous...,????????? ???...
&Clear,Z&urucksetzen,&Effacer,????????
Macro,Makro,Macro,??????
Save As...,Speichern unter...,Enregistrer sous...,????????? ???...
AutoHotkey Scripts,AutoHotkey Scripts,AutoHotkey Scripts?AutoHotkey ???????
Recording macro...`nPress $1 to stop.,Makroaufzeichnung...`nBeenden mit $1,Enregistrement en cours...`nAppuyez sur $1 pour arreter,?????? ???????... `n??? ????????? ??????? $1
"Loading script, please wait...","Lade Skript, bitte warten...","Chargement du script, veuillez patienter..."?"?????? ???????????..."
Save script,Skript speichern,Enregistrer le script,????????? ??????
Would you like to save the current script before closing?,Vor Beenden speichern?,Enregistrer avant de quitter ?,????????? ??????????
Replay,Abspiel,Rejouer,?????????
WAIT!, WARTEN!, ATTENDEZ !, ?????!
Relative to Window, Relative to Fenster, Relative to Fenetre, ?? ????????? ? ????
)

If A_Language in 0407,0807,0c07,1007,1407
   lang = de
Else If A_Language in 040c,080c,0c0c,100c,140c,180c
   lang = fr
Else If A_Language in 0419
   lang = ru
Else lang = en ; default

i = -1
Loop, Parse, lang_table, `n
{
   i++
   Loop, Parse, A_LoopField, CSV
   {
      If i = 0
      {
         If lang = %A_LoopField%
            j = %A_Index%
      }
      Else If A_Index = %j%
      {
         lang%i% = %A_LoopField%
      }
   }
}
;listvars
gui_nogui=0
title = AMPLRecorder
vers = 0.19s
config = %A_ScriptDir%\%title%.ini
config_list = clicks,keyboard,window,ctrl,sleep,movements=0,movint=10,moveinte=10,moverel=0,sleep_hk=F4,window_hk=F5,ctrl_hk=F8,keyboard_hk=F6,clicks_hk=F7,movements_hk=F10,moverel_hk=F11,windowrel_hk=F9,stop=F12,windowrel=0,hideshow_hk=F2,marker=Recording Marker,marker_hk=F3
Loop, Parse, config_list, `,
{
   StringSplit, v, A_LoopField, =
   IniRead, gui_%v1%, %config%, Options, %v1%, % InStr(A_LoopField, "=") ? v2 : 1
}

; Scribbly 2009.09.27
; Parse CommandLine Arguements
if (%0%>0){
  Loop , %0%
  {
    param:=%A_Index%
		StringSplit, v, param, =
		gui_%v1%=%v2%
  }
}

/*
** Script Variables
*/
testScriptPath = %A_Temp%\%title%.temp.ahk
numberRecordings:=0
previous_CoordMode=
numberMarkers:=0

if gui_nogui=0
{
	Gui, +Resize +MinSize +LastFound
	; fixed width font is too big
	Gui, Add, Edit, vscr Hwndhscr w460 h215 gType
	Gui, Font
	gui, Add, GroupBox, vamplGroup y225 xm w180 h45,Insert in AMPL
	Gui, Add, Button, viCodeBefore xp+15 yp+15 w70 Disabled Section gInsertBefore, codeBefore
	Gui, Add, Button, viCodeAfter ys w70 Disabled gInsertAfter, codeAfter
	Gui, Add, GroupBox, vscriptGroup xm+190 y225 w275 h45,Recorded Script
	Gui, Add, Button, vsave Section xp+15 yp+15 w75 Disabled gSave, %lang13%
	Gui, Add, Button, vclear ys w75 Disabled gClear, %lang14%
	Gui, Add, Button, vplay ys w75 Disabled gPlayTemp, %lang22%
	; Recording Options
	Gui, Add, GroupBox, vsec1 Section ym w200 r9.4, %lang2%
	Gui, Add, CheckBox, vgui_sleep xp+15 yp+20 Checked%gui_sleep%, %lang3%
	Gui, Add, CheckBox, vgui_window Checked%gui_window%, %lang4%
	Gui, Add, CheckBox, vgui_keyboard Checked%gui_keyboard%, %lang6%
	Gui, Add, CheckBox, vgui_clicks Checked%gui_clicks%, %lang7%
	Gui, Add, CheckBox, vgui_ctrl xp+40 yp+19 Checked%gui_ctrl% , %lang5%
	Gui, Add, CheckBox, vgui_windowrel Checked%gui_windowrel%, %lang24%
	Gui, Add, CheckBox, vgui_movements xp-40 yp+19 Checked%gui_movements% gMov, %lang8%
	Gui, Add, Edit, vgui_moveinte xp+125 yp-3 w50 Limit4
	Gui, Add, UpDown, vgui_movint Range1-5000 Number, %gui_movint%
	Gui, Add, CheckBox, vgui_moverel xs+55 yp+22 Checked%gui_moverel%, %lang9%
	Gui, Add, Text, vgui_marker_label xp-40 yp+19, Marker
	Gui, Add, Edit, vgui_marker xp+60 yp-3 w120 h18, %gui_marker%
	; Buttons
	Gui, Add, Button, vbutton_help xs yp+30 w95  ggoHelp , Help
	Gui, Add, Button, vbutton_stop xs+105 yp w95  gStop Disabled, %lang12%
	Gui, Add, Button, vlbl3 xs yp+35 w200 Default gStart, %lang11%
	; Hotkeys
	Gui, Add, GroupBox, vsec2 ys w60 r13 Section, %lang10%
	Gui, Add, Hotkey, vgui_sleep_hk xp+5 yp+17 w50 h18 Limit1 , %gui_sleep_hk%
	Gui, Add, Hotkey, vgui_window_hk yp+19 w50  h18  Limit1 , %gui_window_hk%
	Gui, Add, Hotkey, vgui_keyboard_hk yp+19 h18 w50  Limit1 , %gui_keyboard_hk%	
	Gui, Add, Hotkey, vgui_clicks_hk yp+19 h18 w50  Limit1 , %gui_clicks_hk%	
	Gui, Add, Hotkey, vgui_ctrl_hk yp+19 h18 w50  Limit1 , %gui_ctrl_hk%
	Gui, Add, Hotkey, vgui_windowrel_hk yp+19 h18 w50  Limit1 , %gui_windowrel_hk%
	Gui, Add, Hotkey, vgui_movements_hk yp+19 h18 w50  Limit1 , %gui_movements_hk%
	Gui, Add, Hotkey, vgui_moverel_hk yp+19 h18 w50  Limit1 , %gui_moverel_hk%
	Gui, Add, Hotkey, vgui_marker_hk yp+19 h18 w50  Limit1 , %gui_marker_hk%

/*
  2009.09.30 (scribbly): 	remove start and playback hotkeys
                          Start & Playbak uses button
	Gui, Add, Text, vlbl1 xp+15 yp+25, %lang11%:
	Gui, Add, Hotkey, vgui_start xp+40 yp-2 Limit1 gHotkey, %gui_start%
*/
	;Gui, Add, Text, vlbl2 xp-40 yp+27, %lang12%:
	;Gui, Add, Text, vlbl2 xp+15 yp+27, %lang12%:
	Gui, Add, Hotkey, vgui_stop yp+32 h18 w51 Limit1 gHotkey, %gui_stop%
  GUI, font , cBlue s7
  Gui, Add, Text, vgui_hideshow_label yp+22,Show/Hide
  GUI, font , cBlack
	Gui, Add, Hotkey, vgui_hideshow_hk yp+13 h18 w50  Limit1 , %gui_hideshow_hk%
/*
  2009.09.30 (scribbly): 	remove start and playback hotkeys
                          Start & Playbak uses button
	Gui, Add, Text, vlbl4 xp-40 yp+29, %lang22%:
	Gui, Add, Hotkey, vgui_play xp+40 yp-2 Limit1 gHotkey, %gui_play%
*/
	Gui, Show, , %lang1% (v%vers%)
} else {

}

SendMessage, 208, 0, RegisterCallback("EditWordBreakProc"), , ahk_id %hscr%

if gui_nogui=0
	GoSub, Mov

; adapted from Skan's script at http://www.autohotkey.com/forum/viewtopic.php?p=123323#123323:
DllCall("RegisterShellHookWindow", "UInt", WinExist())
OnMessage(DllCall("RegisterWindowMessage", "Str", "SHELLHOOK"), "ShellMessage")

Hotkey:
/*
  2009.09.30 (scribbly): 	remove start and playback hotkeys
                          Start & Playbak uses button
GuiControlGet, v, , gui_start
If v !=
   Hotkey, %v%, Start
GuiControlGet, v, , gui_play
If v !=
   Hotkey, %v%, PlayTemp
*/


recorder_HotKeyList=

if gui_nogui=1
	{
		;ToolTip, %lang23%, 5, 5
		hotkeys(true) ; Metaxal: needs to be BEFORE setting gui_start and gui_stop??
		set_RecorderHotkeys(true)
		StringReplace, s, lang18, $1, %gui_stop%
		StringReplace, s, s, ``n, `n
    Gosub, StartNoGui
	} else {

	}
Return

;;; END OF AUTO-EXECUTE SECTION ;;;

GuiSize:
Anchor("scr", "wh")
v = sec1,gui_movint,gui_clicks,gui_keyboard,gui_window,gui_ctrl,gui_sleep,gui_movements,gui_movint,gui_moverel,sec2,lbl1,gui_start,lbl2,gui_stop,gui_play,lbl4,gui_moveinte,gui_windowrel,gui_clicks_hk,gui_ctrl_hk,gui_hideshow_hk,gui_keyboard_hk,gui_movements_hk,gui_sleep_hk,button_stop,gui_window_hk,gui_windowrel_hk,lbl3,gui_hideshow_label,gui_moverel_hk,button_help,gui_marker,gui_marker_hk,gui_marker_label
Loop, Parse, v, `,
   Anchor(A_LoopField, "x", true)
v =save,amplGroup,icodeBefore,clear,icodeAfter,play,scriptGroup
Loop, Parse, v, `,
   Anchor(A_LoopField, "y", true)
;Anchor("save", "y")
;Anchor("clear", "xy")
;Anchor("lbl3", "xy")
Return

Mov:
GuiControlGet, v, , gui_movements
GuiControl, Enable%v%, gui_moveinte
GuiControl, Enable%v%, gui_moverel
Return

Clear:
;GuiControl, , scr
ControlSend, , ^{Home}^+{End}{Del}, ahk_id %hscr% ; this allows undo
numberRecordings=0
numberMarkers=0
Type:
GuiControlGet, scr, , scr
v := StrLen(scr) > 0
GuiControl, Enable%v%, save
GuiControl, Enable%v%, clear
GuiControl, Enable%v%, play
Gosub, EnableAMPLButtons
unsaved := true
Return
EnableAMPLButtons:
IfWinExist, AMPL :: Step, ,
	v:=1
Else
	v:=0
GuiControl, Enable%v%, icodeBefore
GuiControl, Enable%v%, icodeAfter
Return
Save:
GuiControlGet, scr, , scr
Gui, +OwnDialogs
FormatTime, d, , ShortDate
FormatTime, t, , Time
StringReplace, t, t, :, .
StringReplace, d, d, /, -, All
FileSelectFile, file, S19, %A_MyDocuments%\%lang15% %d% %t%.ahk, %lang13%, %lang17% (*.ahk; *.txt)
If file =
   Return
FileDelete, %file%
FileAppend, %scr%, %file%
unsaved := false
VarSetCapacity(scr, 0)
Return

InsertBefore:
ampl_CodeControl=Edit11
Gosub, InsertIntoAMPL
Return
InsertAfter:
ampl_CodeControl=Edit12
Gosub, InsertIntoAMPL
Return

InsertIntoAMPL:
IfWinNotExist, AMPL :: Step, ,
{
  MsgBox, 0, %title%, AMPL Step window has been closed...`n`n...Try opening it before trying that again?
  Gosub, EnableAMPLButtons
  return
}

ControlGetText, v , %ampl_CodeControl%, AMPL :: Step, ,
if v
	v.="`r`n"
GuiControlGet, scr, , scr
StringReplace, scr, scr, `n, `r`n, All
NewText:=v . scr
ControlSetText , %ampl_CodeControl%, %NewText%, AMPL :: Step, ,
Return
Start:
/*
	Use a button to stop recording and reSet other options on the fly
*/

;ToolTip, %lang23%, 5, 5
hotkeys(true) 		; Metaxal: needs to be BEFORE setting gui_start and gui_stop??
GuiControlGet, v, FocusV
If v in gui_start,gui_stop
   Return
Gui, Submit
guiControlGet, v, , gui_stop
StringReplace, s, lang18, $1, %v%
StringReplace, s, s, ``n, `n
set_RecorderHotkeys(true)     ; Scribbly: this replaces Recorder hotkeys in the above
GuiControlGet, gui_movint, , gui_moveinte
StartNoGui:
if gui_windowrel ; Not yet functional!
   CoordMode, Mouse, Relative
else
   CoordMode, Mouse, Screen
if gui2Created!=1
{
Gui,2: -Caption +Border +AlwaysOnTop +ToolWindow
Gui,2: Color, Silver
Gui,2: +LastFound
gui,2: font, s7, Verdana
Gui,2:  Add, Button, x+5 yp w75 gStop, Stop (%gui_stop%)
Gui,2: Add, CheckBox, vrec_sleep xp+80 yp+5 Section Checked%gui_sleep% gToggleSleep, Time (%gui_sleep_hk%)
Gui,2: Add, CheckBox, vrec_window xp+110 ys Checked%gui_window% gToggleWindow, Windows (%gui_window_hk%)
Gui,2: Add, CheckBox, vrec_keyboard xp+110 ys  Checked%gui_keyboard% gToggleKeyBoard, Keyboard (%gui_keyboard_hk%)
Gui,2: Add, CheckBox, vrec_clicks xp+110 ys  Checked%gui_clicks% gToggleClicks, Clicks (%gui_clicks_hk%)
Gui,2: Add, CheckBox, vrec_ctrl xp+110 ys  Checked%gui_ctrl% gToggleCtrl, Controls (%gui_ctrl_hk%)
Gui,2: Add, CheckBox, vrec_windowrel xp+110 ys Checked%gui_windowrel% gToggleWindowRel, Window Rel (%gui_windowrel_hk%)
Gui,2: Add, CheckBox, vrec_movements xp+110 ys  Checked%gui_movements% gToggleMovements, Movements (%gui_movements_hk%)
Gui,2: Add, CheckBox, vrec_moverel xp+110 ys  w120  Checked%gui_moverel% gToggleMoveRel, Relative (%gui_moverel_hk%)
gui2Created=1
} else {
GuiControl,2: , rec_sleep , %gui_sleep%
GuiControl,2: , rec_window , %gui_window%
GuiControl,2: , rec_ctrl , %gui_ctrl%
GuiControl,2: , rec_keyboard , %gui_keyboard%
GuiControl,2: , rec_clicks , %gui_clicks%
GuiControl,2: , rec_movements , %gui_movements%
GuiControl,2: , rec_moverel , %gui_moverel%
GuiControl,2: , rec_windowrel , %gui_windowrel%
}
Gui,2:  Show, x1 y1 h22 NoActivate, Recording Window                  ; Adjust X & Y to suit your screen res
recorderWindow_Visible:=1
Gui,2: +LastFound
Gui,2: default
WinGet, recorder_winID, ID, Recording Window
log_mpx := 0
log_mpy := 0
log_s = 1
log_xt := A_TickCount ; Initialize sleep counter
last_cmd=
log_text=
numberRecordings+=1
if numberRecordings=1
{
	add_log_text("#NoEnv") ; SendMode Input`n" ; buggy with mousemove! (not synchronous)
	add_log("CoordMode", "Mouse, " . (gui_windowrel ? "Relative" : "Screen"))
	add_log("SetMouseDelay",  "2")
if gui_ctrl
   add_log("SetControlDelay","-1") 										; Use SetControlDelay -1 prior to ControlClick. This avoids holding
	 																										; the mouse button down during the click, which in turn reduces
																											; interference from the user's physical movement of the mouse.
}

add_log_text("`n; -------------------------------`n; Recording: " .  numberRecordings )
if numberRecordings>1
	add_log_text("; NOTE: Will not run unless previous 'ExitApp' line is removed")
add_log_text("; -------------------------------" )
If (gui_movements and gui_movint > 0 and gui_movint <= 5000) {
   SetTimer, MouseMov, %gui_movint%
   }
Gui, Color, EEAA99
;ToolTip, %s%, 5, 5

;Loop {
;   Input, v, L1 V ; now useless
;   logkey(v)
;   If !log_s
;      Break
;}

Return

ToggleSleep:
	gui_sleep:= gui_sleep=0 ? 1 : 0
	GuiControl,2: , rec_sleep , %gui_sleep%
	add_log_text("`n; Toggle Time Intervals [" . gui_sleep . "]")
	lastFunctionKey:=ConvertToVK(gui_sleep_hk)
Return
ToggleWindow:
	gui_window:= gui_window=0 ? 1 : 0
	GuiControl,2: , rec_window , %gui_window%
	add_log_text("`n; Toggle Window Actions [" . gui_window . "]")
	lastFunctionKey:=ConvertToVK(gui_window_hk)
Return
ToggleCtrl:
	gui_ctrl:= gui_ctrl=0 ? 1 : 0
	GuiControl,2: , rec_ctrl , %gui_ctrl%
	add_log_text("`n; Toggle Control Actions [" . gui_ctrl . "]")
	lastFunctionKey:=ConvertToVK(gui_ctrl_hk)
Return
ToggleKeyBoard:
	gui_keyboard:= gui_keyboard=0 ? 1 : 0
	GuiControl,2:  , rec_keyboard , %gui_keyboard%
	add_log_text("`n; Toggle Keyboard Actions [" . gui_keyboard . "]")
	lastFunctionKey:=ConvertToVK(gui_keyboard_hk)
Return
ToggleClicks:
	gui_clicks:= gui_clicks=0 ? 1 : 0
	GuiControl,2:  , rec_clicks , %gui_clicks%
	add_log_text("`n; Toggle Mouse Click Actions [" . gui_clicks . "]")
	lastFunctionKey:=ConvertToVK(gui_clicks_hk)
Return
ToggleMovements:
	gui_movements:= gui_movements=0 ? 1 : 0
	GuiControl,2:  , rec_movements , %gui_movements%
	add_log_text( "`n; -------------------------------`n; Toggle Mouse Movements [" . gui_movements . "]`n; -------------------------------")
	gui_movements ? (SetTimer, MouseMov , On) : (SetTimer, MouseMov , Off)
	lastFunctionKey:=ConvertToVK(gui_movements_hk)
Return
ToggleMoveRel:
	gui_moverel:= gui_moverel=0 ? 1 : 0
	GuiControl,2:  , rec_moverel , %gui_moverel%
	add_log_text("`n; -------------------------------`n; Toggle Move Relative [" . gui_moverel . "]`n; -------------------------------")
	lastFunctionKey:=ConvertToVK(gui_moverel_hk)
	gui_windowrel ? (CoordMode, Mouse, Relative) : (CoordMode, Mouse, Screen)
  MouseGetPos, x, y , this_winID
  log_mpx := x,log_mpy := y
Return
ToggleWindowRel:
	gui_windowrel:= gui_windowrel=0 ? 1 : 0
	GuiControl,2:  , rec_windowrel , %gui_windowrel%
	add_log_text("`n; -------------------------------`n; Toggle Window Relative [" . gui_windowrel . "]`n; -------------------------------")
	if gui_windowrel=0
	{
		add_log("CoordMode", "Mouse, Screen")
    CoordMode, Mouse , Screen
	} else {
		add_log("CoordMode", "Mouse, Relative")
    CoordMode, Mouse , Relative
	}
	lastFunctionKey:=ConvertToVK(gui_windowrel_hk)
  MouseGetPos, x, y
  log_mpx := x,log_mpy := y
Return
InsertScriptMarker:
	numberMarkers+=1
	add_log_text("`n; -------------------------------`n; " . gui_marker . " :: " . numberMarkers . "`n; -------------------------------")
Return
ToggleRecorderWindow:
	if recorderWindow_Visible=1
	{
	  gui, 2: Hide
	  recorderWindow_Visible:=0
	} else
	{
		gui, 2: Show, NoActivate
		recorderWindow_Visible:=1
	}
	lastFunctionKey:=ConvertToVK(gui_hideshow_hk)
  ;MsgBox, %lastFunctionKey%
Return
Stop:
log_s = 0
SetTimer, MouseMov, Off
hotkeys(false)
set_RecorderHotkeys(false)
gui,2:  Hide
if gui_nogui=0
{
	Gui 1: Default
	Gui,1: Show
  GUI,1: +LastFound
	ToolTip
	GuiControlGet, v, , scr
	If v
	   v .= "`n"
	;log_text .= "`nExitApp`n" . gui_stop . "::ExitApp`n" ; the exit key is the one used to stop the script!
	add_log_text("`nExitApp")
	FileDelete %testScriptPath%
	FileAppend %log_text%, %testScriptPath%
	GuiControl,1: , scr, %v%%log_text%
	VarSetCapacity(log_text, 0)
	ControlSend, , ^{End}, ahk_id %hscr%
	Goto, Type
} else
{
	ToolTip
/*
def_MoveDo=0
def_MoveX=0
def_MoveY=0
def_SizeDo=0
def_SizeW=0
def_SizeH=0
def_delayBefore=0
def_delayAfter=0
def_minmaxAfter=0
def_stepTitle=
def_cmdPath=
def_cmdParameter=
def_cmdProcess=
def_codeBefore=
def_cmdRunWait=0
def_codeAfter=
def_MinMaxAfter=1
def_waitFor=1
def_waitForTimeOut=0
def_actionWhat=1
def_actionWhen=1
def_actionForce=0
def_actionTimeOut=0
def_CmdNone=0
def_msgShow=0
def_msgTimeOut=0
def_winTitle=
def_winText=
def_winExTitle=
def_winExText=
def_winClass=
def_GetFileParameter=
def_LargeIcon=
def_iconPath=
def_iconListViewNumber=1
*/
commandList:=CompileCommandObject( "Recorded Step" , "" , 0 , "" , "", 0, "" , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 1 , 1 , 0 , 1 , 1 , 0 , 0 , "", log_text , "" , "" , 1 , 1, 1 , 0, 0)

scriptName:= gui_scriptName="" ? RecordedScript : gui_scriptName
SplitPath, scriptname , OutFileName, OutDir, OutExtension, OutNameNoExt, OutDrive
scriptName.= OutExtension="" ? ".ahk" : ""
iniName=RecordedScript.ini

scriptPath = %ampldirectory%\scripts\%scriptName%
iniPath = %ampldirectory%\scripts\%iniName%
includePath = %ampldirectory%\scripts\includes\AMPL_Includes.ahk

  ;msgbox, Generate %scriptName% in or around %directoryAMPL%`nScript Path: %scriptPath%`nInclude Path: %includePath%
if(FileExist(scriptPath)<>"") {
  MsgBox, 4, %title% , %scriptName% Exists! Did you want to overwrite it?
  IfMsgBox, Yes
	{
    FileDelete, %scriptPath%
    IfExist, %iniPath%
      FileDelete, %iniPath%
		scriptHasBeenWritten:=1
	}
	else
	{
	  Return
	}
}

WriteFiles(scriptPath , iniPath , includePath, False )
run_ahk="%A_AhkPath%"
run_se="%ampldirectory%\ScriptEditor\ScriptEditor.ahk"
run_script="%scriptName%"
ampldirectory="%ampldirectory%"
;MsgBox, %run_ahk%`n%run_se%`n%run_script%`n%ampldirectory%
Run, %run_ahk%  %run_se% %run_script% %ampldirectory%
ExitApp

}
Return

LogKey:
If log_s {
   StringTrimLeft, v, A_ThisHotkey, 2 ; -2 to remove "~*"
   logkey(v)
}
Return

MouseMov:
	hideMouseClick:=""
  Gosub, RecordMouseMovement
Return
RecordMouseMovement:
	if (gui_windowrel!=previous_CoordMode)
	{
	 gui_windowrel ? (CoordMode, Mouse, Relative) : (CoordMode, Mouse, Screen)
	 previous_CoordMode:=gui_windowrel
	}
   MouseGetPos, x, y , this_winID
   if (recorder_winID=this_winID)
   	return
	 ;showToolTip("MouseMov: " . x . ":" . y)
   If (x != log_mpx or y != log_mpy) {
      If gui_moverel
         add_log( hideMouseClick . "MouseMove ", (log_mpx ? x - log_mpx : 0) . ", " . (log_mpy ? y - log_mpy : 0) . ",, R")
      Else
         add_log(hideMouseClick . "MouseMove", x . ", " . y)
   }

   log_mpx := x
   log_mpy := y
Return
goHelp:
IfExist, AMPL Recorder.chm
  Run, AMPL Recorder.chm
else
  MsgBox, Can't find the Help file!
Return
PlayTemp:
	GuiControlGet, v, , scr
	FileDelete %testScriptPath%
	FileAppend %v%, %testScriptPath%

	Run %A_AhkPath% %testScriptPath%
Return

;~*Esc:: ; for debugging
GuiClose:
GuiControlGet, scr, , scr
If (scr != "" and unsaved)
{
   Gui, +OwnDialogs
   MsgBox, 35, %lang20%, %lang21%
   IfMsgBox, Yes
      Gosub, Save
   IfMsgBox, Cancel
    Return
}
Gui, Submit
Loop, Parse, config_list, `,
{
   StringSplit, v, A_LoopField, =
   IniWrite, % gui_%v1%, %config%, Options, %v1%
}
ExitApp

; Metaxal
add_log(cmd, params) {
   global last_cmd, log_text, log_xt, gui_sleep, log_xt

   If gui_sleep and (cmd != "Sleep") {
      v := Round((A_TickCount - log_xt) / 1)
      If (log_text and v > 700)
         add_log("Sleep", v) ;
      log_xt := A_TickCount
   }

   If (SubStr(cmd, 1, 4) = "Send") and (cmd = last_cmd)
      log_text .= params
   Else
      log_text .= "`n" . cmd . ", " . params

   last_cmd = %cmd%
}
add_log_text(stringToAdd) {
  global log_text

	log_text .= stringToAdd . "`n"

}

logkey(v) {
   local _w, _ct

	 MouseGetPos, OutputVarX, OutputVarY, OutputVarWin, OutputVarControl, 1
   if (recorder_winID=OutputVarWin)
   	return

   k := SubStr(v, 1, 4)
   if v not contains up
   {
      v = %v% down
			controlClick_Direction=D
	 } else {
			controlClick_Direction=U
	 }
	 
	 if v=%lastFunctionKey% up
	 {
		;log_text .= "`n; Skipped leftover " . v
	  lastFunctionKey:=""
    Return 
		}
   v = {%v%}

   ; VK codes can also detect mouse down and up !!
   if k in VK01,VK02,VK04 ; mouse clicks
   {
			if k=VK01
				mouseButton := "LEFT"
			else if k=VK02
        mouseButton :=  "RIGHT" 
			else
			 mouseButton :="MIDDLE"
      
      if (gui_ctrl and OutputVarControl) {
         ;showToolTip("Mouse click on Control: " . OutputVarControl . "(" . mouseButton . ")")
         cc:=  OutputVarControl . ",,," . mouseButton . ",1,NA " . controlClick_Direction
         add_log("ControlClick", cc) ;
         hideMouseClick:="; "
         prev_controlClick_Direction:=controlClick_Direction,prev_outputVarControl:=OutputVarControl,prev_mouseButton:=mouseButton
			} else {
         hideMouseClick:=""
         prev_controlClick_Direction=
			}
			If gui_clicks {
				if (controlClick_Direction=U and prev_controlClick_Direction=D)
				{
         cc:= prev_OutputVarControl . ",,," . prev_mouseButton . ",1,NA " . "U"
         add_log("; Automatic closing of ControlClick", "") ;				
         add_log("ControlClick", cc) ;		
				 hideMouseClick:="; "		
				} else if hideMouseClick=
				{
         hideMouseClick:=""
        }

				GoSub RecordMouseMovement
				;showToolTip("MouseClick: " . v)
				add_log( hideMouseClick . "Send", v) ; Synchronous click
      }
   }
   Else If (gui_keyboard && v!=last_Keyboard) { ; Scribbly: v!=last_Keyboard stops key repeat
      add_log("SendInput", v)
      last_Keyboard:=v
   }
}

ShellMessage(wParam, lParam) {
   local t, c
   ;If wParam = 4 ; HSHELL_WINDOWACTIVATED
   If (wParam = 32772) or (wParam = 4) ; HSHELL_RUDEAPPACTIVATED or HSHELL_WINDOWACTIVATED
   {
      ; Why not just use active window here??
      ; This is not working with PSPad: get a different ID and Class!!
      ;WinGetTitle, t, ahk_id %lParam%
      WinGetTitle, t, A
      ;WinGetClass, c, ahk_id %lParam%
      WinGetClass, c , A
			if t=Recording Window
			  return
      If (log_s and gui_window and t and c) {
         d := t . " ahk_class " . c
         ;add_log("WinWait", d)
         add_log("IfWinNotActive", d . ", , WinActivate, " . d)
         add_log("WinWaitActive", d)
      }
   }
}

hotkeys(on = true) {

   ; VK codes seem to be much more reliable concerning different keyboard configurations.
   ; also handles properly the down and up of several keys, even mouse ones.
   ; also handles joysticks ?!

   ; This should allow for *any* foreign keyboard (?)
   ; although it is not very readable, so the script cannot really be modified manually
   nums=0123456789ABCDEF
   StringSplit numarray, nums

   Loop %numarray0%
   {
      n1 := numarray%A_Index%
      Loop %numarray0%
      {
         n2 := numarray%A_Index%
         n := n1 . n2
         if n=00
            continue
         k := "*~VK" . n
         Hotkey, %k%, LogKey, % "B" . (on ? "On" : "Off") ;%
         Hotkey, %k% up, LogKey, % "B" . (on ? "On" : "Off") ;%
      }
   }

   bugkeys=Delete,Home,PgUp,PgDn,Insert,End,NumpadEnter,Up,Down,Left,Right
   Loop, Parse, bugkeys, `,
   {
      k =  ~*%A_LoopField%
      Hotkey, %k%, LogKey, % "B" . (on ? "On" : "Off") ;%
      Hotkey, %k% up, LogKey, % "B" . (on ? "On" : "Off") ;%
   }

}

; taken from http://www.autohotkey.com/forum/topic20337.html
EditWordBreakProc(lpch, ichCurrent, cch, code) {
   static exp = "\W" ; treat any non alphanumeric character as a delimiter with this regex
   Loop, % cch * 2 ; build the string: ;%
      str .= Chr(*(lpch - 1 + A_Index))
   If code = 0 ; WB_LEFT
   {
      ; reverse and return last match:
      StringMid, str, str, 1, --ichCurrent
      Loop, Parse, str
         rev := A_LoopField . rev
      Return, ichCurrent - RegExMatch(rev, exp, "[^" . exp . "]+$") + 1
   }
   Else If code = 1 ; WB_RIGHT
      Return, ichCurrent >= cch ? cch : RegExMatch(str, exp, "", ichCurrent + 1) - 1
   Else If code = 2 ; WB_ISDELIMITER
      Return, RegExMatch(SubStr(str, ichCurrent, 1), exp)
}
showToolTip(m) {
  ToolTip, %m%, 5, 25
}
ConvertToVK(s) {
  ;MsgBox, %s%
	if s=F1
	  Return, "VK70"
	if s=F2
	  Return, "VK71"
	if s=F3
	  Return, "VK72"
	if s=F4
	  Return, "VK73"
	if s=F5
	  Return, "VK74"
	if s=F6
	  Return, "VK75"
	if s=F7
	  Return, "VK76"
	if s=F8
	  Return, "VK77"
	if s=F9
	  Return, "VK78"
	if s=F10
	  Return, "VK79"
	if s=F11
	  Return, "VK7A"
	if s=F4
	  Return, "VK73"
}
set_RecorderHotkeys(on = true) {
	global gui_stop, gui_sleep_hk, gui_window_hk, gui_windowrel_hk, gui_ctrl_hk, gui_keyboard_hk, gui_clicks_hk, gui_movements_hk, gui_moverel_hk,gui_windowrel_hk,gui_hideshow_hk,gui_marker_hk

If gui_stop !=
   Hotkey, %gui_stop%, Stop, % "B" . (on ? "On" : "Off")
If gui_sleep_hk !=
   Hotkey, %gui_sleep_hk%, ToggleSleep, % "B" . (on ? "On" : "Off")
If gui_window_hk !=
   Hotkey, %gui_window_hk%, ToggleWindow, % "B" . (on ? "On" : "Off")
If gui_ctrl_hk !=
   Hotkey, %gui_ctrl_hk%, ToggleCtrl, % "B" . (on ? "On" : "Off")
If gui_keyboard_hk !=
   Hotkey, %gui_keyboard_hk%, ToggleKeyBoard, % "B" . (on ? "On" : "Off")
If gui_clicks_hk !=
   Hotkey, %gui_clicks_hk%, ToggleClicks, % "B" . (on ? "On" : "Off")
If gui_movements_hk !=
   Hotkey, %gui_movements_hk%, ToggleMovements, % "B" . (on ? "On" : "Off")
If gui_moverel_hk !=
   Hotkey, %gui_moverel_hk%, ToggleMoveRel, % "B" . (on ? "On" : "Off")
If gui_windowrel_hk !=
   Hotkey, %gui_windowrel_hk%, ToggleWindowRel, % "B" . (on ? "On" : "Off")
If gui_hideshow_hk !=
   Hotkey, %gui_hideshow_hk%, ToggleRecorderWindow, % "B" . (on ? "On" : "Off")
if gui_marker_hk!=
   Hotkey, %gui_marker_hk% , InsertScriptMarker, % "B" . (on ? "On" : "Off")
}
