; AMPL Include Functions
version_AMPL_Includes=0.4.1
version_AMPL_Package=0.6.2

/*
Release History
	0.4.x
	  todo: Add error variable for final action (ampl_ERROR=Success=0|Error<100|Fail>100)
	  todo: Reuse existing window if it exists?
	  todo: First log line should be something like 200909061010 ---------- Script Run ----------
	  todo: Set timeout error level for WinWaitClose?
	0.4.1 2009.09.14
	  ! fixed bug in FinalAction
	0.4.0 2009.07.02
	  + WinText, WinExTitle, WinExText
	0.3.3 2009.06.22
	  ^ Used GoSubs in ProcessCommand::GetWindowID section
	  ! Fixed false Error in waitForWindow when winID="" and waitFor=1 (nothing)
	0.3.2 2009.06.12
	  + Full Logging
	  ~ Minor corrections to window management
	0.3.1
	  + TimeOut changed to variables
	  + WorkingDirectory
	  ! Fixed postProcess
	0.3.0 2009.05.06
	  + Extended actionWhat
  0.2.0 2009.04.28
    + onError added to FinalAction
	0.1.5 2009.04.20
	  ! Use window for all actions
	  ~ Return PID & ID for later use
	0.1.4 2009.03.28
	  ~ Added a combined Move & Do in one step
  0.1.3: 29/01/2009
    + Added working directory
*/

/*
	ERROR Levels
	This script will set ERRORLEVEL as follows:
	0 			= Success
	>1      = WARN
	>100    = FAIL
*/
; Version Check
; version_AMPL_Includes_Used is written in the script file
if (version_AMPL_Includes_Used < version_AMPL_Includes)
{
  StringSplit, versionWrittenBy, version_AMPL_Includes_Used, . ,
  StringSplit, versionCurrent, version_AMPL_Includes, .,
  if versionWrittenBy1 < %versionCurrent1%
    {
		  MsgBox, Your Script needs to be updated to use this version of AMPL`nAMPL Includes: %version_AMPL_Includes%`nYour Script: %version_AMPL_Includes_Used%
    	;msgbox, %versionWrittenBy1% < %versionCurrent1%
		  ExitApp, 500
    } else if versionWrittenBy2 < %versionCurrent2%
    {
		  MsgBox, Your Script needs to be updated to use this version of AMPL`nAMPL Includes: %version_AMPL_Includes%`nYour Script: %version_AMPL_Includes_Used%
    	;msgbox, %versionWrittenBy1% < %versionCurrent1%
		  ExitApp, 500
    } else if versionWrittenBy3 < %versionCurrent3%
    {
      MsgBox, 1, AMPL, Your Script was written using an older version of AMPL`nAMPL Includes: %version_AMPL_Includes%`nYour Script: %version_AMPL_Includes_Used%`nDid you want to continue or Cancel and update your scripts?, 60
    	;msgbox, %versionWrittenBy1% < %versionCurrent1%
      IfMsgBox, Cancel
        ExitApp, 1
    }
}
DoLogToFile=False
iniLocationNotDefined =
SplitPath, A_SCRIPTFULLPATH , scriptFullName, scriptDirectory, scriptExt, scriptName, scriptDrive
SetWorkingDir, %A_SCRIPTDIR%
scriptIni = %A_SCRIPTDIR%\%scriptName%.ini
scriptLog = %A_SCRIPTDIR%\%scriptName%.log


ampl_FindPath(fileToRun, fileName){
	Global scriptIni, scriptDrive

	if SubStr(fileToRun, 2, 1) <> ":"
		{
	    ; fileToRun is on same drive as script!
			Log("Adding Drive [" . scriptDrive . "]: " . fileToRun)
		  fileToRun = %scriptDrive%\%fileToRun%
		}

	If FileExist( fileToRun )<>%iniLocationNotDefined%
		{
		  Log("File Exists: [" . fileToRun . "]")
			Return fileToRun
		}
	else {
	  Log("File Does Not Exist: [" . fileToRun . "]")
		LOOP {
	    ; Look through INI locations
			Key = AltLocation%A_INDEX%

	    IniRead, OutputVar, %scriptIni%, %fileName%, %Key%
			Log("ReadIni " . fileName . "," . Key . "=[" . OutputVar . "]")
	    if( OutputVar <> "ERROR" and OutputVar <> "" ) {
	      Log("Checking " . Key )
				if( FileExist(OutputVar)<>%iniLocationNotDefined%) {
					  Log("File Exists: [" . OutputVar . "]")
				    Return %OutputVar%
	          Break
					} else {
						Log("File Does not Exist: [" . OutputVar . "]")
					}
			}
			else {
	      SplitPath, fileToRun, , , OutExtension, OutName
	      ;msgbox, Looking for %OutName%.%OutExtension%
	      Log("Requesting new location for [" . OutName . "." . OutExtension . "]")
	      FileSelectFile, SelectedFile, 3, , Find the file, Same File (*%OutName%.%OutExtension%  )
		    ;msgbox, Write to %scriptIni%: %fileName% %Key%: %SelectedFile%
	      if(SelectedFile<>""){
	        Log("File Selected: [" . SelectedFile . "]")
	      	IniWrite, %SelectedFile%, %scriptIni%, %fileName%, %Key%
	      	Log("Writing INI: " . scriptIni . " Section: " . fileName . " Key: " . Key . " Value: " . SelectedFile)
	        ;MsgBox, Error: %ERRORLEVEL%
	      	Return %SelectedFile%
	      } else {
					Log("No File Selected")
				}
				Break
			} ; End Else
		}	; End LOOP
	} ; End Else
	; If we get here there is an error and we can't run the command
	Log("Setting ErrorLevel = 1")
	ERRORLEVEL = 1
}

ampl_ProcessCommand( cmdTitle , cmdName, PreDelay , cmdPath, cmdParams, cmdWkDir, useRunWait, nameProcess , PostDelay , MoveDo , MoveX , MoveY , SizeDo , SizeW , SizeH, PostProcess, winTitle, winClass, winText, winExTitle, winExText, msgShow, msgTimeOut, waitPostRun, WinWaitPID,WinWaitCLASS,WinWaitTITLE){
Global DoLogToFile, scriptLog

Log("Processing Command: [" . cmdTitle  . "] : [" . cmdName . "] : [" . PreDelay  . "] : [" . cmdPath . "] : [" . cmdParams . "] : [" . cmdWkDir . "] : [" . useRunWait . "] : [" . nameProcess  . "] : [" . PostDelay  . "] : [" . MoveDo  . "] : [" . MoveX  . "] : [" . MoveY  . "] : [" . SizeDo  . "] : [" . SizeW  . "] : [" . SizeH . "] : [" . PostProcess . "] : [" . winTitle . "] : [" . winClass . "] : [" . WinText . "] : [" . WinExTitle . "] : [" . WinExText . "] : [" . msgShow . "] : [" . msgTimeOut . "] : [" . waitPostRun . "] : [" . WinWaitSeconds . "]")
	ampl_winID=
	ampl_procID=
	pc_Error:=0

	if(cmdPath=""){
	  Log("Command Path is BLANK")
		ERRORLEVEL=100
	  Log("ErrorLevel set to " . ERRORLEVEL)
	  Return
		}
	Log("Getting New Command for [" . cmdPath . "]")
	NewcmdPath := ampl_FindPath(cmdPath, cmdName)

	if(NewcmdPath=""){
	  Log("New Command Path is BLANK")
		ERRORLEVEL=101
	  Log("ErrorLevel set to " . ERRORLEVEL)
	  Return
		}
	; Begining the script
	Log("Sleep: [" . PreDelay . "]")
	Sleep, %PreDelay%
	SplitPath, NewcmdPath,, fileDir
	if cmdWkDir=
	  {
	  	cmdWkDir:=fileDir
			Log("Set cmdWkDir:=[" . fileDir . "]")
	  }
	if useRunWait=0
  {
    Log("Run: [" . NewcmdPath . "] [" . cmdParams . "] cmdWkDir: [" . cmdWkDir . "]" )
		Run, %NewcmdPath% %cmdParams%, %cmdWkDir% , UseErrorLevel , ampl_procID
		if ERRORLEVEL=ERROR
		{
		  ERRORLEVEL=102
		  Log("Error on Run: " . A_LastError)
		  Log("ErrorLevel set to " . ERRORLEVEL)
		  Return
		}
	  Log("PostRun Sleep: [" . waitPostRun . "]")
    Sleep, %waitPostRun%
    findStart:=A_TICKCOUNT
		if ampl_procID!=
		{
		  Log("ProcessID returned: [" . ampl_procID . "]")
		  ; Wait for the window from the process
			if winClass!=
        Gosub, WaitPIDClassTitle
			else
        Gosub, WaitPIDTitle
		} else
		{
			Log("No ProcessID Returned")
			if winClass!=
			  Gosub, WaitClassTitle
			else
        Gosub, WaitTitle
		}
		;then if ampl_winID=not set then skip the rest of the window commands
		if ampl_winID!=
		{
			findDuration:=A_TICKCOUNT - findStart
		  Log("Found window after " . findDuration . " milliseconds")
		  Log("Using WindowID: [" . ampl_winID . "]")
	    WinGet, ampl_procID_check , PID, ahk_ID %ampl_winID%
		  Log("WinGet PID " . ampl_winID . " =[" . ampl_procID_check . "]")
		  if ampl_procID!=%ampl_procID_check%
		  {
					Log("ProcessID changed from value returned from RUN: " . ampl_procID . "->" . ampl_procID_check)
					Log("Changing ProcessID to [" . ampl_procID_check . "]")
					ampl_procID:=ampl_procID_check
			}
			; Check the state of the window
			; If it's Maximised we'll restore it so that any change is kept correctly by windows
			WinGet, WinState, MinMax, ahk_id %ampl_winID%
			Log("WinGet MinMax ahk_id " . ampl_winID . "=[" . WinState . "]")
			if WinState=
			{
				pc_Error+=1
				Log("!! Error!! pc_Error=" . pc_Error)
			}
			else
			{
				if WinState<>0
				{
				  Log("WinRestore ahk_id " . ampl_winID)
					WinRestore, ahk_id %ampl_winID%
					if ERRORLEVEL
					  {
							pc_Error+=1
							Log("!! Error!! pc_Error=" . pc_Error)
						}
				}
			}
			;If we're Moving and Sizing we'll do it in one commande
			if (MoveX="" or MoveY="")
			  {
        	WinGetPos, currentX, currentY, currentW, currentH, ahk_id %ampl_winID%,
			    Log("Using existing window coortinates: x:" . currentX . " y:" . currentY . " w:" . currentW . " h:" . currentH)
        	if MoveX=
        	{
        	  Log("MoveX<-" . currentX)
        	  MoveX:=currentX
					}
					if MoveY=
					{
					  Log("MoveY<-" . currentY)
					  MoveY:=currentY
					}
					if SizeW=
					{
					  Log("SizeW<-" . currentW)
					  ;SizeW:=currentW
					}
					if SizeH=
					{
					  Log("SizeH<-" . currentH)
						;SizeH:=currentH
					}
				}
			if (MoveDo=1 AND SizeDo=1)
			{
			  Log("WinMove ahk_id " . ampl_winID . " x:" . MoveX . " y:" . MoveY . " w:" . SizeW . " h:" . SizeH)
				WinMove, ahk_id %ampl_winID%, , %MoveX%, %MoveY%, %SizeW% , %SizeH%
				if ERRORLEVEL
			   {
						Log("!! Error!! pc_Error=" . pc_Error)
					  Log("WinMove " . winTitle . " x:" . MoveX . " y:" . MoveY . " w:" . SizeW . " h:" . SizeH)
						WinMove, %winTitle%, , %MoveX%, %MoveY%, %SizeW% , %SizeH%
						if ERRORLEVEL
						{
							Log("!! Error!! pc_Error=" . pc_Error)
							pc_Error+=1
						}
			   }
			} else
			{
				if MoveDo=1
				{
				  Log("WinMove ahk_id " . ampl_winID . " x:" . MoveX . " y:" . MoveY )
					WinMove, ahk_id %ampl_winID%, , %MoveX%, %MoveY%
				}
				if ERRORLEVEL
				{
					Log("!! Error!! pc_Error=" . pc_Error)
				  Log("WinMove " . winTitle . " x:" . MoveX . " y:" . MoveY )
				  WinMove, %winTitle%, , %MoveX%, %MoveY%
					if ERRORLEVEL
					{
						Log("!! Error!! pc_Error=" . pc_Error)
						pc_Error+=1
					}
					; Size the window
	   		}
				if SizeDo=1
				{
				  Log("WinMove ahk_id " . ampl_winID . " w:" . SizeW . " h:" . SizeH)
					WinMove, ahk_id %ampl_winID%, , , , %SizeW% , %SizeH%
				}
				if ERRORLEVEL
				{
					Log("!! Error!! pc_Error=" . pc_Error)
				  Log("WinMove " . winTitle . " w:" . SizeW . " h:" . SizeH)
					WinMove, %winTitle%, , , , %SizeW% , %SizeH%
					if ERRORLEVEL
						pc_Error+=1
						Log("!! Error!! pc_Error=" . pc_Error)
				}
			}
			if PostProcess > 1
			{
				if PostProcess=2
				{
				  Log("WinMinimize ahk_id " . ampl_winID)
			    WinMinimize, ahk_id %ampl_winID%
					if ERRORLEVEL
					  {
						  if (msgShow=1)
				        MsgBox, Can't Minimise window!
							pc_Error+=1
							Log("!! Error!! pc_Error=" . pc_Error)
						}
			  } else if PostProcess=3
			  {
			    ;msgbox, WinMaximize, ahk_id %ampl_winID%
				  Log("WinMaximize ahk_id " . ampl_winID)
			    WinMaximize, ahk_id %ampl_winID%
					if ERRORLEVEL
					{
					  if (msgShow=1)
			        MsgBox, Can't Maximise window!
						pc_Error+=1
						Log("!! Error!! pc_Error=" . pc_Error)
					}
			  }
			}
		} else
		{
		  if (msgShow=1)
        MsgBox, Can't get Window ID!
		   pc_Error+=102
		  Log("Can't get WindowID: pc_Error=" . pc_Error)
			findDuration:=A_TICKCOUNT - findStart
		  Log("Waited for " . findDuration . " milliseconds")
		}
	} else
  {
    Log("RunWait: [" . NewcmdPath . "] [" . cmdParams . "] cmdWkDir: [" . cmdWkDir . "]" )
    RunWait, %NewcmdPath% %cmdParams%, %cmdWkDir% , UseErrorLevel ,
			if ERRORLEVEL=ERROR
			{
			  ERRORLEVEL=102
		  	Log("Error on RunWait: " . A_LastError)
			}
		ampl_procID=
	  pc_Error+=ERRORLEVEL     
  }

	Log("Sleep: [" . PostDelay . "]")
	Sleep, %PostDelay%
	
	; END of the Script
	
	; Return the aggrigated ErrorLevels
	Log("Setting ErrorLevel=[" . pc_Error . "]")
	ERRORLEVEL:=pc_Error
	; Return the IDs so that we can use it later
	Log("Setting Return Value=[" . ampl_procID . "~" . ampl_winID . "]")
	valueReturn:=ampl_procID . "~" . ampl_winID
	Return valueReturn

WaitPIDClassTitle:
  Log("WinWait: TITLE=[" . winTitle . "] TEXT=[" . WinText . "] PID=[" . ampl_procID . "] CLASS=[" . winClass . "] TimeOut=[" . WinWaitPID . "] EXCLUDE_TITLE=[" . WinExTitle . "] EXCLUDE_TEXT=[" . WinExText . "]")
  waitStart:=A_TICKCOUNT
  WinWait, %winTitle% ahk_pid %ampl_procID% ahk_class %winClass%, %WinText% , %WinWaitPID%, %WinExTitle% , %WinExText%
  if ERRORLEVEL
	{
		waitDuration:=A_TICKCOUNT - waitStart
	  Log("WinWait Timed Out after " . waitDuration . " milliseconds")
    Gosub, WaitPIDClass
	} else {
		waitDuration:=A_TICKCOUNT - waitStart
	  Log("WinWait Found window after " . waitDuration . " milliseconds")
		WinGet, ampl_winID , ID, %winTitle% ahk_pid %ampl_procID% ahk_class %winClass%
    Log("WinGet ID TITLE=[" . winTitle . "] ahk_pid=[" . ampl_procID . "] CLASS=[" . winClass . "] := [" . ampl_winID . "]")
	}
Return
WaitPIDClass:
  Log("WinWait: PID=[" . ampl_procID . "]  CLASS=[" . winClass . "] TimeOut=[" . WinWaitPID . "] EXCLUDE_TITLE=[" . WinExTitle . "] EXCLUDE_TEXT=[" . WinExText . "]")
  waitStart:=A_TICKCOUNT
  WinWait, ahk_pid %ampl_procID% ahk_class %winClass%, , %WinWaitPID%, %WinExTitle% , %WinExText%
  if ERRORLEVEL=0
  {
		waitDuration:=A_TICKCOUNT - waitStart
	  Log("WinWait Found window after " . waitDuration . " milliseconds")
		WinGet, ampl_winID , ID, ahk_pid %ampl_procID% ahk_class %winClass%
    Log("WinGet ID ahk_pid=[" . ampl_procID . "] CLASS=[" . winClass . "] := [" . ampl_winID . "]")
	} else {
		waitDuration:=A_TICKCOUNT - waitStart
	  Log("WinWait Timed Out after " . waitDuration . " milliseconds")
	  ; At this point it can be assumed that the PID returned by Run is not the PID that ran the window
	  Log("?? Either the time waited is too short: try increasing the Wait PID Window variable [var_WinWaitPIDSeconds] or the PID that created the window is different to the one returned from the Run Command (this can be checked with the [Check Window] function on the Window Tab, if this is the case leave the wait period=0.")
    Gosub, WaitClassTitle
	}
Return
WaitPIDTitle:
  Log("WinWait: TITLE=[" . winTitle . "] TEXT=[" . WinText . "] PID=[" . ampl_procID . "]  TimeOut=[" . WinWaitPID . "] EXCLUDE_TITLE=[" . WinExTitle . "] EXCLUDE_TEXT=[" . WinExText . "]")
  waitStart:=A_TICKCOUNT
  WinWait, %winTitle% ahk_pid %ampl_procID%,%WinText% , %WinWaitPID%, %WinExTitle% , %WinExText%
  if ERRORLEVEL
  {
		waitDuration:=A_TICKCOUNT - waitStart
	  Log("WinWait Timed Out after " . waitDuration . " milliseconds")
    Gosub, WaitPID
  } else {
	  Log("WinWait Found window after " . waitDuration . " milliseconds")
		WinGet, ampl_winID , ID, %winTitle% ahk_pid %ampl_procID% a
    Log("WinGet ID TITLE=[" . winTitle . "] ahk_pid=[" . ampl_procID . "] := [" . ampl_winID . "]")
	}
Return
WaitPID:
  Log("WinWait: PID=[" . ampl_procID . "]  TimeOut=[" . WinWaitPID . "] EXCLUDE_TITLE=[" . WinExTitle . "] EXCLUDE_TEXT=[" . WinExText . "]")
  waitStart:=A_TICKCOUNT
  WinWait, ahk_pid %ampl_procID%, , %WinWaitPID%, %WinExTitle% , %WinExText%
  if ERRORLEVEL=0
  {
	  Log("WinWait Found window after " . waitDuration . " milliseconds")
		WinGet, ampl_winID , ID, ahk_pid %ampl_procID% a
    Log("WinGet ID ahk_pid=[" . ampl_procID . "] := [" . ampl_winID . "]")
	} else {
		waitDuration:=A_TICKCOUNT - waitStart
	  Log("WinWait Timed Out after " . waitDuration . " milliseconds")
    Gosub, WaitTitle
	}
Return
WaitClassTitle:
  Log("WinWait: Title=[" . winTitle . "] TEXT=[" . WinText . "] CLASS=[" . winClass . "] TimeOut=[" . WinWaitCLASS . "] EXCLUDE_TITLE=[" . WinExTitle . "] EXCLUDE_TEXT=[" . WinExText . "]")
  waitStart:=A_TICKCOUNT
	WinWait, %WinTitle% ahk_class %winClass%,%WinText% , %WinWaitCLASS%, %WinExTitle% , %WinExText%
	if ERRORLEVEL
	{
		waitDuration:=A_TICKCOUNT - waitStart
	  Log("WinWait Timed Out after " . waitDuration . " milliseconds")
    Gosub, WaitClass
	} else {
		waitDuration:=A_TICKCOUNT - waitStart
	  Log("WinWait Found window after " . waitDuration . " milliseconds")
    WinGet, ampl_winID , ID, %WinTitle% ahk_class %winClass%,%WinText% , %WinExTitle% , %WinExText%
    Log("WinGet ID TITLE=[" . winTitle . "] ahk_class=[" . winClass . "]:=[" . ampl_winID . "]")
	}
Return
WaitClass:
  Log("WinWait: CLASS=[" . winClass . "] TimeOut=[" . WinWaitCLASS . "] EXCLUDE_TITLE=[" . WinExTitle . "] EXCLUDE_TEXT=[" . WinExText . "]")
  waitStart:=A_TICKCOUNT
	WinWait, ahk_class %winClass%, , %WinWaitCLASS%, %WinExTitle% , %WinExText%
	if ERRORLEVEL
	{
		waitDuration:=A_TICKCOUNT - waitStart
	  Log("WinWait Timed Out after " . waitDuration . " milliseconds")
	  Log("!! The CLASS setting [" . winClass . "] seems to be incorrect: please check with the [Check Window] button")
	  ; At this point we have not got a window based on previous CLASS settings, so there's been a change
	  ; We'll try again down the PID+Title avenue
	  if ampl_procID!=
    	Gosub, WaitPIDTitle
		else
      Gosub, WaitTitle
	} else {
		waitDuration:=A_TICKCOUNT - waitStart
		; Check Process is the same here as nameProcess
	  Log("WinWait Found window after " . waitDuration . " milliseconds")
    WinGet, ampl_winID , ID, ahk_class %winClass%, , %WinExTitle% , %WinExText%
    Log("WinGet ID TITLE=[" . winTitle . "] ahk_class=[" . winClass . "]:=[" . ampl_winID . "]")
	}
Return
WaitTitle:
  Log("WinWait: Title=[" . winTitle . "] TEXT=[" . WinText . "] TimeOut=[" . WinWaitTITLE . "] EXCLUDE_TITLE=[" . WinExTitle . "] EXCLUDE_TEXT=[" . WinExText . "]")
  waitStart:=A_TICKCOUNT
  WinWait, %WinTitle% , %WinText%, %WinWaitTITLE%, %WinExTitle% , %WinExText%
  if ERRORLEVEL
  {
		waitDuration:=A_TICKCOUNT - waitStart
	  Log("WinWait Timed Out after " . waitDuration . " milliseconds")
	} else {
		waitDuration:=A_TICKCOUNT - waitStart
	  Log("WinWait Found window after " . waitDuration . " milliseconds")
    WinGet, ampl_winID , ID, %WinTitle%, %WinText%, %WinExTitle% , %WinExText%
    Log("WinGet ID TITLE=[" . winTitle . "] := [" . ampl_winID . "]")
	}
Return
FoundWindow:
	waitDuration:=A_TICKCOUNT - waitStart
  Log("WinWait Found window after " . waitDuration . " milliseconds")
	WinGet, ampl_winID , ID, ahk_pid %ampl_procID%
  Log("WinGet ID ahk_pid " . ampl_procID . "=[" . ampl_winID . "]")
Return
}

ampl_WaitForWindow( thisID , timeOut , waitFor ){
	valueReturn=0
	Log("Wait For Window: ID:[" . thisID . "] TimeOut:[" . timeOut . "] waitFor:[" . waitFor . "]")
	;msgbox,WinID: %thisID%`nWaitFor: %waitFor%`nTimeOut: %timeOut%
	if thisID= and waitFor>1
	{
		; Can't process window, return a Fail
		Log("!! Error: Can't process blank ID")
		Log("Setting ErrorLevel=100")
		ERRORLEVEL=100
	  Return
	}

	DetectHiddenWindows, On
	Log("DetectHiddenWindows, On")
		if timeOut=0
			timeOut:=""
		if (waitFor=2) {
		  Log("WinWaitNotActive ahk_id " . thisID . " timeOut: " . timeOut)
	    WinWaitNotActive, ahk_id %thisID%, , %timeOut%
	    valueReturn:=ERRORLEVEL
	    if valueReturn
				Log("!! Error!! pc_Error=" . valueReturn)
		} else if(waitFor=3) {
		  Log("WinWaitClose ahk_id " . thisID . " timeOut: " . timeOut)
	    WinWaitClose, ahk_id %thisID%, , %timeOut%
	    valueReturn:=ERRORLEVEL
	    if valueReturn
				Log("!! Error!! pc_Error=" . valueReturn)
		} else {
			Log("Nothing to wait for...")
		}
	DetectHiddenWindows, off
	Log("DetectHiddenWindows, off")
	Log("Setting ErrorLevel:=" . valueReturn)
	ERRORLEVEL:=valueReturn
}

ampl_FinalAction( actionWhen, stepError , actionWhat , actionForce , actionTimeOut , actionWindow )
{
	Log("Final Action: [" . actionWhen . "] : [" . stepError . "] : [" . actionWhat . "] : [" . actionForce . "] : [" . actionTimeOut . "] : [" . actionWindow . "]")
  ;MsgBox, When: %actionWhen%`nSuccess: %stepError%`nWhat: %actionWhat%`nForce: %actionForce%`nTimeOut: %actionTimeOut%`nWindow: %actionWindow%
	doAction=0
	actionCode=0
	actionType=0
	/*
	Action What Codes:
		Nothing					1
		Normalize       2
		Minimize Window 3
		Maximize        4
		Close Window    5
		Minimize All    6
		Log Off         7
		Shut Down       8
		Reboot          9
		Power Down      10
 */
 /*
  Action When Codes
    Always          1
    After Success   2
    After Error     3
    After Failure   4
 */
  ; Do we need to do anything?
  ; 2009.07.19 PM: added parenthesis
	if (actionWhen=1 and actionWhat>1)
	  {
	  	doAction=1
	  	Log("actionWhen=1 and actionWhat>1: doAction=1")
	  }
	else if (actionWhen=2 and stepError=0)
	  {
	  	doAction=1
	  	Log("actionWhen=2 and stepError=0: doAction=1")
	  }
  else if (actionWhen=3 and stepError>0 and stepError<100)
	  {
	  	doAction=1
	  	Log("actionWhen=3 and stepError>0 and stepError<100: doAction=1")
	  }
	else if (actionWhen=4 and stepError>=100)
	  {
	  	doAction=1
	  	Log("actionWhen=4 and stepError>=100: doAction=1")
	  }
  ;MsgBox, DoAction= %doAction%
  Log("doAction=" . doAction)
	if doAction=1
	{
		if actionWhat=1     ; Nothing
		  {
		  	actionType=0      ; Don't do anything
		  	Log("actionWhat=1: actionType=0")
		  }
		if (actionWhat>1 and actionWhat<7)
		  {
		  	actionType=2      ; Window Action
		  	Log("actionWhat>2 and actionWhat<7: actionType=2")
			}
		if actionWhat=7     ; LogOff
		  {
		  	actionType=1      ; System Action
		  	actionCode=0
				actionName=LogOff
		  	Log("actionWhat=7: actionType=1; actionCode=7; actionName=LogOff")
			}
		if actionWhat=8     ; ShutDown
		  {
		  	actionType=1      ; System Action
		  	actionCode=1
				actionName=ShutDown
		  	Log("actionWhat=8: actionType=1; actionCode=8; actionName=ShutDown")
			}
		if actionWhat=9     ; ReBoot
		  {
		  	actionType=1      ; System Action
		  	actionCode=2
		  	actionName=ReBoot
		  	Log("actionWhat=9: actionType=1; actionCode=2; actionName=ReBoot")
			}
		if actionWhat=10     ; PowerDown
		  {
		  	actionType=1      ; System Action
		  	actionCode=9
		  	actionName=ShutDown and ReBoot
		  	Log("actionWhat=10: actionType=1; actionCode=9; actionName=hutDown and ReBoot")
			}

		if actionForce=1
		  {
		  	actionCode+=4
		  	Log("actionForce=1: actionCode+=4 :=[" . actionCode . "]")
		  }
		  
		;msgbox, ahk_id: %actionWindow%`nType: %actionType%
		if actionType=1
		  {
		    Log("Confirmaton of " . actionName)
        MsgBox, 1, AMPL, This computer is about to %actionName%`nSelect Cancel if you do not want this to happen, %actionTimeOut%
        IfMsgBox, Cancel
        {
          Log("User Cancelled " . actionName)
          Return, "I don't want to shutdown"
				}
				Log("Shutdown " . actionCode)
      	Shutdown, %actionCode%
			} else if actionWindow!=
			{
				if actionType = 2
				{
				  if actionWhat=3
					{
					  Log("WinMinimize ahk_id " . actionWindow)
						WinMinimize, ahk_id %actionWindow%
					} else if actionWhat = 2
					{
					  Log("WinRestore ahk_id " . actionWindow)
	        	WinRestore, ahk_id %actionWindow%
						WinGet, WinState, MinMax, ahk_id %actionWindow%
						Log("WinGet MinMax ahk_id" . actionWindow . "=[" . WinState . "]")
						if WinState=
						  {
								;MsgBox, Error Getting 'WinGet'
								Log("!! Error!! ErrorLevel=" . ERRORLEVEL)
							}
						else
						{
							if WinState<>0
							{
							  Log("PostMessage 0x112, 0xF120,,, ahk_id " . actionWindow)
		        	  PostMessage, 0x112, 0xF120,,, ahk_id %actionWindow%  ; 0x112 = WM_SYSCOMMAND, 0xF030 = SC_MAXIMIZE
								if ERRORLEVEL
								  {
										pc_Error+=1
										Log("!! Error!! ErrorLevel=" . ERRORLEVEL)
									}
							}
						}
					} else if actionWhat = 5
					{
					  Log("WinClose ahk_id " . actionWindow)
	        	WinClose, ahk_id %actionWindow%
	        	if (ERRORLEVEL and actionForce=1)
	        	  {
								Log("!! Error!! ErrorLevel=" . ERRORLEVEL)
							  Log("WinKill ahk_id " . actionWindow)
                WinKill, ahk_id %actionWindow%
							}
					} else if actionWhat = 4
					{
					  Log("WinMaximize ahk_id " . actionWindow)
	        	WinMaximize, ahk_id %actionWindow%
						WinGet, WinState, MinMax, ahk_id %actionWindow%
						Log("WinGet MinMax ahk_id" . actionWindow . "=[" . WinState . "]")
						if WinState=
						  {
								MsgBox, Error Getting 'WinGet'
								Log("!! Error!! ErrorLevel=" . ERRORLEVEL)
							}
						else
						{
							if WinState<>1
							{
							  Log("PostMessage 0x112, 0xF030,,, ahk_id " . actionWindow)
		        	  PostMessage, 0x112, 0xF030,,, ahk_id %actionWindow%  ; 0x112 = WM_SYSCOMMAND, 0xF030 = SC_MAXIMIZE
								if ERRORLEVEL
								  {
										pc_Error+=1
										Log("!! Error!! ErrorLevel=" . ERRORLEVEL)
									}
							}
						}
					} else if actionWhat = 6
					{
					  Log("WinMinimizeAll")
            WinMinimizeAll
					}
				}
			}
	} else {
		Log("Nothing to do...")
	}
}
Log(LogString)
{
  global
	;MsgBox, Log: %scriptLog%`nDoLog: %DoLogToFile%`n%LogString%
  if (DoLogToFile = True)
  {
      FormatTime, TimeString , , hh:mm:ss
      FileAppend, %TimeString% %LogString%`n,%scriptLog%
      Return , ERRORLEVEL
  }
}
